----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 07/02/2019 06:16:26 AM
-- Design Name: 
-- Module Name: FSM_controller - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity FSM_controller is
    Port ( clk : in STD_LOGIC;
           rst : in STD_LOGIC;
           start : in STD_LOGIC;
           tx_ready : in STD_LOGIC;
           rst_busy : in STD_LOGIC;
           uart_tx_we : out STD_LOGIC;
           ROM_en : out STD_LOGIC;
           addr : out STD_LOGIC_VECTOR(3 downto 0)
           );
end FSM_controller;

architecture Behavioral of FSM_controller is

constant DLY : time := 1 ns;
type rom_state is (wait4start, check_ready, en_cnt_addr, en_rom, end_msg);
signal state, next_state : rom_state;

signal ROM_count : unsigned (3 downto 0) :=(others=>'0');
signal ROM_count_en : std_logic :='0';
signal next_ROM_count_en : std_logic :='0';
signal next_ROM_en : std_logic :='0';
signal ROM_end_msg : std_logic :='0';
signal ROM_en_i : std_logic :='0';



begin

addr <= std_logic_vector(ROM_count);

reg_porc: process (clk, rst) 
begin
  if (rst = '1') then
    ROM_count <= (others => '0') after DLY;
    uart_tx_we <='0' after DLY;
  elsif rising_edge(clk) then
    
    if ROM_count_en ='1' then        
        if (ROM_count = 9) then
          ROM_count <= (others => '0') after DLY;
        else
          ROM_count <= ROM_count + 1 after DLY;
        end if;
    end if;
    
    uart_tx_we  <= ROM_en_i after DLY;
    
  end if;
  end process;

process (rst, clk)
begin  
    if (rst = '1') then
        state           <= wait4start after DLY;
        ROM_count_en    <= '0' after DLY;
        ROM_en_i          <= '0' after DLY;
    elsif rising_edge(clk) then
        state           <= next_state after DLY;
        ROM_count_en    <= next_ROM_count_en after DLY;
        ROM_en_i         <= next_ROM_en after DLY;
    end if;    
end process;  
      
      
comb_proc: process(state, start, tx_ready, rst_busy, ROM_count)      
begin  
    case state is
        when wait4start =>
            if (start = '1') then
                next_state <= check_ready; 
                next_ROM_count_en <='0'; 
                next_ROM_en <='0'; 
            else
                next_state <= wait4start;
                next_ROM_count_en <='0';
                next_ROM_en <='0'; 
            end if;
            
          when check_ready =>
            if ROM_count =9 then
                next_state <= end_msg;
                next_ROM_count_en <='0';
                next_ROM_en <='0';
--            if ROM_end_msg = '1' then
--                next_state <= end_msg;
--                next_ROM_count_en <='0';
--                next_ROM_en <='0'; 
            elsif (tx_ready = '1' and rst_busy ='0') then
              next_state <= en_cnt_addr;  
              next_ROM_count_en <='1';
              next_ROM_en <='0'; 
            else
              next_state <= check_ready;
              next_ROM_count_en <='0';
              next_ROM_en <='0'; 
          end if;

          when en_cnt_addr =>
            next_state <= en_rom;
            next_ROM_count_en <='0';
            next_ROM_en <='1'; 
         
            when en_rom =>
                next_state <= check_ready;
                next_ROM_count_en <='0';
                next_ROM_en <='0';
            
          when end_msg =>
              next_state <= end_msg;
              next_ROM_count_en <='0'; 
              next_ROM_en <='0';
          
           when others => 
            next_state <= wait4start;
            next_ROM_count_en <='0'; 
            next_ROM_en <='0';
        end case;
   end process;

ROM_en <= ROM_en_i;

end Behavioral;
