----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 07/01/2019 07:08:52 PM
-- Design Name: 
-- Module Name: UART_rx - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity TEST_UART_tx is
Generic ( 
simulation : boolean := TRUE--FALSE--
);
Port ( clk : in STD_LOGIC;
       rst : in STD_LOGIC;
       start : in STD_LOGIC;
       uart_tx : out STD_LOGIC;
       LEDs : out STD_LOGIC_VECTOR(3 downto 0)
);
end TEST_UART_tx;

architecture Behavioral of TEST_UART_tx is


component LEDs_controller is
Generic ( 
simulation : boolean := TRUE--FALSE--
);
Port ( 
    clk : in STD_LOGIC;--100 MHz oscillator (E3)
    rst : in STD_LOGIC;
    LEDs : out std_logic_vector(3 downto 0)
);
end component;

COMPONENT msg_rom
  PORT (
    clka : IN STD_LOGIC;
    rsta : IN STD_LOGIC;
    ena : IN STD_LOGIC;
    addra : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
    douta : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    rsta_busy : OUT STD_LOGIC
  );
END COMPONENT;


component debouncer is
Generic ( 
simulation : boolean := TRUE--FALSE--
);
Port (                   
  clk                 		: in    std_logic;
  signal_in                 : in    std_logic;
  signal_out                : out   std_logic
);
end component;

component FSM_controller is
    Port ( clk : in STD_LOGIC;
           rst : in STD_LOGIC;
           start : in STD_LOGIC;
           tx_ready : in STD_LOGIC;
           rst_busy : in STD_LOGIC;
           uart_tx_we : out STD_LOGIC;
           ROM_en : out STD_LOGIC;
           addr : out STD_LOGIC_VECTOR(3 downto 0)
           );
end component;

component uart_tx6 is
  Port (             data_in : in std_logic_vector(7 downto 0);
                en_16_x_baud : in std_logic;
                  serial_out : out std_logic;
                buffer_write : in std_logic;
         buffer_data_present : out std_logic;
            buffer_half_full : out std_logic;
                 buffer_full : out std_logic;
                buffer_reset : in std_logic;
                         clk : in std_logic);
  end component;
  
  
COMPONENT ila_uart
  
  PORT (
      clk : IN STD_LOGIC;
  
  
  
      probe0 : IN STD_LOGIC_VECTOR(6 DOWNTO 0); 
      probe1 : IN STD_LOGIC_VECTOR(0 DOWNTO 0);
      probe2 : IN STD_LOGIC_VECTOR(0 DOWNTO 0)
  );
  END COMPONENT  ;  

signal en_rom : std_logic:='0';
signal rst_busy : std_logic:='0';
signal addr : std_logic_vector( 3 downto 0):=(others=>'0');
signal dout : std_logic_vector( 7 downto 0):=(others=>'0');

signal sync_start : std_logic:='0';

signal baud_count : integer range 0 to 108 := 0; 		
signal en_16_x_baud : std_logic := '0';		

signal buffer_write : std_logic := '0';	
signal buffer_data_present : std_logic := '0';	
signal buffer_half_full : std_logic := '0';	
signal buffer_full : std_logic := '0';	
signal buffer_reset : std_logic := '0';	

signal uart_tx_we: std_logic := '0';	 
signal tx_ready : std_logic := '0';	
signal tx_not_ready : std_logic := '0';	

begin



blinker_inst: LEDs_controller
Generic map( 
    simulation => simulation
)
Port map ( 
    clk  => clk,
    rst  => rst,
    LEDs => LEDs
);


msg_rom_inst: msg_rom
PORT MAP (
    clka => clk,
    rsta => rst,
    ena => en_rom,
    addra => addr,
    douta => dout,
    rsta_busy => rst_busy
);
  
  
debouncer_inst: debouncer
Generic map( 
    simulation => simulation
)
Port map(                   
    clk => clk,
    signal_in => start,
    signal_out  => sync_start
); 

FSM_controller_inst: FSM_controller
Port map( 
    clk => clk,
    rst => rst,
    start => sync_start,
    tx_ready => tx_ready,
    rst_busy => rst_busy,
    uart_tx_we => uart_tx_we,
    ROM_en  => en_rom,
    addr => addr
); 

baud_rate: process(clk)	
begin	
   if clk'event and clk = '1' then	
        if baud_count = 53 then                    	
               baud_count <= 0;	
               en_16_x_baud <= '1';	
        else	
               baud_count <= baud_count + 1;	
               en_16_x_baud <= '0';	
        end if;	
   end if;	
end process baud_rate;	
 
 
 uart_tx_inst: uart_tx6
  Port map(             
                     data_in => dout,
                en_16_x_baud => en_16_x_baud,
                  serial_out => uart_tx,
                buffer_write => uart_tx_we,
         buffer_data_present => open,
            buffer_half_full => open,
                 buffer_full => tx_not_ready,
                buffer_reset => rst,
                         clk => clk
                         );

tx_ready <= not (tx_not_ready);
  
 
ila_inst: ila_uart
PORT MAP (
	clk => clk,
	probe0 => dout, 
	probe1(0) => uart_tx_we,
	probe2(0) => tx_not_ready
);
  
end Behavioral;
